/*******************************************************************************
 *
 * htmltag.c - HTML parsing engine. 
 *                   
 * These routines parse the the individual html tags and store the 
 * attributes into a linked list.
 * 
 * Cheetah Web Browser
 * Copyright (C) 2001 Garett Spencley 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
 *
 *******************************************************************************/

#include <stdlib.h>
#include <string.h>

#include "error.h"
#include "htmltag.h"
#include "htmlparser.h"
#include "string_t.h"

/*
 * create_tag_args() 
 *
 * creates a linked list of arguments contained
 * within tag
 */

html_tag_args *create_tag_args(char *string)
{
	char *name;
	html_tag_args *current;
	html_tag_args *args;
	html_tag_args *link;
	string_t *buf;

	if(!string)
		return NULL;

	buf = create_string(string);
	if(!buf) {
		error("out of memory.");
		return NULL;
	}

	/* Fill in the first node */

	name = get_ident(buf);

	if(name) {
			
		args = allocate_tag_args();

		args->name = strdup(name);
		if(!args->name) {
			error("out of memory.");
			free(name);
			free_string(buf);
			return NULL;
		}
		free(name);
		
		/* If an equal sign follows then this 
		   argument has a value. Otherwise set it to NULL */

		if(get_char(buf) != '=') 
			args->value = NULL;
		else {
		
			args->value = get_string(buf);
			if(!args->value) {
				
				args->value = get_ident(buf);
				if(!args->value) {
					error("Bad HTML syntax. 2");
					free(args->name);
					free_string(buf);
					return NULL;
				}
			}
		}

	} else {
		free_string(buf);
		return NULL;
	}
	
	/* Create the rest of the nodes */

	link = args;

	while((name = get_ident(buf))) {
	
		current = allocate_tag_args();
		if(!current) {
			error("Out of memory.");
			free(name);
			free_string(buf);
			return NULL;
		}

		current->name = strdup(name);
		if(!current->name) {
			error("Out of memory.");
			free(name);
			free_string(buf);
			return NULL;
		}
		free(name);
		
		/* If an equal sign follows then this 
		   argument has a value. Otherwise set it to NULL */

		if(get_char(buf) != '=')
			current->value = NULL;
		else {
			current->value = get_string(buf);
			if(!current->value) {

				current->value = get_ident(buf);
				if(!current->value) {
					error("Bad HTML syntax. 4");
					free(current->name);
					free_string(buf);
					return NULL;
				}
			}
		}

		link->next = current;
		link = link->next;

		current = current->next;
	}

	free_string(buf);
	return args;
}

/*
 * allocate_tag_args()
 *
 * allocates a linked list for tag arguments
 */

html_tag_args *allocate_tag_args()
{
	html_tag_args *result;

	result = (html_tag_args *)malloc(sizeof(html_tag_args));
	if(!result)
		return NULL;

	result->next = NULL;
	
	return result;
}

/*
 * free_tag_args() 
 *
 * frees an html_tag_args list
 */

void free_html_tag_args(html_tag_args *args)
{
	html_tag_args *current;
	html_tag_args *next;

	current = args;

	while(current) {

		free(current->name);
		free(current->value);

		next = current->next;

		free(current);

		current = next;
	}
}
